(defproject report "0.1.0-SNAPSHOT"
  :description "Give a report of the yas snippets"
  :dependencies [[org.clojure/clojure "1.10.0"]
                 [hiccup "1.0.5"]]

  :main core
  :uberjar-name "report.jar"
  :min-lein-version "2.8.1"
  :source-paths ["src"]
  :test-paths ["test"]
  :resource-paths ["resources"])
